"use strict";
const Pager = require('./pager');

module.exports = function (domain, user, password) {
  var url = 'https://' + domain + '.myspringboard.us/api/';
  var request = require('./rest-request')(url, user, password);
  var filterPager = {};
  var sprinboardApi = {};
  sprinboardApi.request = request;


  // Item
  //   id: integer(required)
  //   cost: numeric(10, 2)(required) The default cost of purchasing this item
  //   price: numeric(10, 2)(required) The default sales price
  //   public_id: text The unique public identifier for this item. This can be used to provide your own naming scheme to your inventory. Items can be searched for using this value.
  //   metadata: text a placeholder for custom data (deprecated)
  //   description: text A short description of the item.
  //   long_description: text A long description of the item
  //   custom: hash An object that acts as a container for custom values defined in Springboard Retail as well as any arbitrary values stored by integrations or custom applications.
  //   active: Boolean Whether or not this record is currently active and being used
  //   created_at: DateTime The date and time that this record was created
  //   updated_at: DateTime The date and time that this record was last updated
  //   financial_class_id: integer The id of the financial class this item belongs to.
  //   import_batch_id: integer If this item was imported or modified by an import this attribute references the import batch that brought this item into the system or updated it most recently.
  //   primary_vendor_id: integer The id for the primary vendor record that supplies this item.

  sprinboardApi.createItem = function (cost, price, data, callback) {
    data = data || {};
    data.cost = cost;
    data.price = price;
    request('items', 'postJson', data, callback);
  };

  sprinboardApi.getItem = function (itemId, callback) {
    request('items/' + itemId, 'get', {}, callback);
  };

  sprinboardApi.getItems = function (callback) {
    request('items', 'get', {}, callback);
  };

  sprinboardApi.updateItem = function (data, callback) {
    data = data || {};
    request('items/' + itemId, 'putJson', data, callback);
  };

  sprinboardApi.getItem = function (id, callback) {
    request('items/' + id + '?_include[]=primary_vendor&_include[]=vendor_details', 'get', {}, callback);
  };

  sprinboardApi.getItemByLookup = function (lookup, callback) {
    request('items/lookup_code=' + lookup, 'get', {}, callback);
  };

  sprinboardApi.getItemsByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getItemsByFilterRaw, convertArgsToArray(arguments));
  };

  function getItemsByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('items?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };

  sprinboardApi.getItemVendors = function (id, callback) {
    request('items/' + id + '/vendors', 'get', {}, callback);
  };

  sprinboardApi.createItemVendor = function (itemId, vendorId, data, callback) {
    data = data || {};
    data.item_id = itemId;
    data.vendor_id = vendorId;
    request('items/' + itemId + '/vendors', 'postJson', data, callback);
  };

  sprinboardApi.searchItems = function (searchTerm, page, perPage, callback) {
    data = { 'query': searchTerm.replace(' ', '+'), 'page': page, 'per_page': perPage };
    request('items', 'get', data, callback);
  };


  // Order
  //   id: integer(required)
  //   customer_id: integer(required)
  //   station_id: integer(required)
  //   source_location_id: integer(required)
  //   status: text The status of the order.Valid options: "pending", "open", "canceled", "closed"
  //   sales_rep: text
  //   recalculate: boolean
  //   created_at: DateTime The date and time that this record was created
  //   updated_at: DateTime The date and time that this record was last updated
  //   shipping_method_id: integer
  //   shipping_charge: numeric(10, 2)
  //   billing_address_id: integer
  //   shipping_address_id: integer

  sprinboardApi.createOrder = function (customerId, locationId, stationId, data, callback) {
    var data = data || {};
    data.customer_id = customerId;
    data.source_location_id = locationId;
    data.station_id = stationId;
    request('sales/orders', 'postJson', data, callback);
  };

  sprinboardApi.getOrder = function (orderId, callback) {
    request('sales/orders/' + orderId, 'get', {}, callback);
  };

  sprinboardApi.updateOrder = function (orderId, data, callback) {
    var data = data || {};
    request('sales/orders/' + orderId, 'putJson', data, callback);
  };

  sprinboardApi.createOrderShipping = function (orderId, shippingMethodId, shippingCharge, data, callback) {
    var data = data || {};
    data.shipping_method_id = shippingMethodId;
    data.shipping_charge = shippingCharge;
    request('sales/orders/' + orderId, 'putJson', data, callback);
  };

  sprinboardApi.getOrdersByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getOrdersByFilterRaw, convertArgsToArray(arguments));
  };

  function getOrdersByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('sales/orders?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };


  // OrderLine
  //   id: integer(required)
  //   item_id: integer(required)
  //   qty: numeric(10, 2)(required)
  //   description: text
  //   adjusted_unit_price: numeric(10, 2)
  //   original_unit_price: numeric(10, 2)
  //   unit_cost: numeric(10, 2)
  //   total_tax: numeric(10, 2) Extended tax amount to collect for the line
  //   created_at:DateTime The date and time that this record was created
  //   updated_at: DateTime The date and time that this record was last updated
  //   qty_open: numeric(10, 2)
  //   qty_invoiced: numeric(10, 2)
  //   order_id: integer
  //   ship_from_location_id: integer

  sprinboardApi.createOrderLine = function (orderId, itemId, quantity, data, callback) {
    var data = data || {};
    data.item_id = itemId;
    data.qty = quantity;
    request('sales/orders/' + orderId + '/lines', 'postJson', data, callback);
  };

  sprinboardApi.getOrderLines = function (orderId, callback) {
    request('sales/orders/' + orderId + '/lines', 'get', {}, callback);
  };

  sprinboardApi.shipOrderLines = function (orderId, orderLineId, shippingLocationId, callback) {
    var data = data || {};
    data.ship_from_location_id = shippingLocationId;
    request('sales/orders/' + orderId + '/lines/' + orderLineId, 'putJson', data, callback);
  };


  // Sales Order Payment
  //    id: integer(required)
  //    type: text(required) The payment type.Valid options: ["CashPayment", "CreditCardPayment", "CheckPayment", "CustomPayment", "ExternalPayment"]
  //    deposit: boolean(required) If true, the funds are captured immediately.Otherwise funds are captured when fulfilling the order.
  //    amount:numeric(10, 2)(required)The amount of money to pay.
  //    status: text The payment 's current status indicating whether or not it has been captured.
  //    created_at: DateTime The date and time that this record was created
  //    updated_at: DateTime The date and time that this record was last updated

  sprinboardApi.getOrderPayment = function (orderId, callback) {
    request('sales/orders/' + orderId + '/payments', 'get', {}, callback);
  };

  sprinboardApi.createOrderPayment = function (orderId, data, callback) {
    var data = data || {};
    request('sales/orders/' + orderId + '/payments', 'postJson', data, callback);
  };

  sprinboardApi.getPaymentTypes = function (callback) {
    request('payment_types', 'get', {}, callback);
  };


  // Invoice
  //    id: integer(required)
  //    type: text(required)
  //    metadata: text
  //    customer_id: integer
  //    source_location_id: integer
  //    sales_rep: text
  //    created_at: DateTime The date and time that this record was created
  //    updated_at: DateTime The date and time that this record was last updated
  //    public_id: text
  //    station_id: integer
  //    parent_transaction_id: integer
  //    recalculate: boolean
  //    status: text
  //    order_id: integer
  //    total: numeric(10, 2)
  //    created_by_user_id: integer

  sprinboardApi.createInvoice = function (data, callback) {
    var data = data || {};
    request('sales/invoices', 'postJson', data, callback);
  };

  sprinboardApi.getInvoice = function (invoiceId, callback) {
    request('sales/invoices/' + invoiceId, 'get', {}, callback);
  };

  sprinboardApi.getInvoiceLines = function (invoiceId, callback) {
    request('sales/invoices/' + invoiceId + '/lines', 'get', {}, callback);
  };

  sprinboardApi.createInvoiceLines = function (invoiceId, itemId, qty, type, callback) {
    var data = data || {};
    data.item_id = itemId;
    data.qty = qty;
    data.type = type;
    request('sales/invoices/' + invoiceId + '/lines', 'postJson', data, callback);
  };

  sprinboardApi.updateInvoice = function (invoiceId, data, callback) {
    var data = data || {};
    request('sales/invoices/' + invoiceId, 'putJson', data, callback);
  };

  sprinboardApi.getInvoicesByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getInvoicesByFilterRaw, convertArgsToArray(arguments));
  };

  function getInvoicesByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('sales/invoices?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };


  // Customer
  //   id: integer(required)
  //   public_id: text Unique ID string
  //   first_name: text First name
  //   last_name: text Last name
  //   email: text Email address
  //   address_id: integer ID of default address record
  //   shipping_address_id: integer ID of address record used for shipping
  //   billing_address_id:integer ID of address record used for billing
  //   active:BooleanWhether or not this record is currently active and being used
  //   customer_import_batch_id: integer ID of batch if thisrecordwascreated as part of an import batch
  //   custom: object Custom field values
  //   created_at: DateTime The date and time that this record was created
  //   updated_at: DateTime The date and time that this record was last updated

  sprinboardApi.createCustomer = function (data, callback) {
    var data = data || {};
    request('customers', 'postJson', data, callback);
  };

  sprinboardApi.getCustomer = function (customerId, callback) {
    request('customers/' + customerId, 'get', {}, callback);
  };

  sprinboardApi.updateCustomer = function (customerId, data, callback) {
    var data = data || {};
    request('customers/' + customerId, 'putJson', data, callback);
  };

  sprinboardApi.getCustomersByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getCustomersByFilterRaw, convertArgsToArray(arguments));
  };

  function getCustomersByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('customers?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };


  // Address
  //    id: integer
  //    address_id: integer
  //    current_revision_id: integer
  //    created_at: DateTime The date and time that this record was created
  //    updated_at: DateTime The date and time that this record was last updated
  //    type: text
  //    first_name: text
  //    last_name: text
  //    line_1: text
  //    line_2: text
  //    city: text
  //    state: text If the country is set to 'US' then 2 letter state acronym required
  //    phone: text
  //    country: text Set to 2 letter country acronym.Set value to anything other than 'US' to use a non standard state / postal_code
  //    postal_code: text If the country is set to 'US' then a 5 digit ZIP or ZIP + 4 code is required
  //    custom: object Custom field values

  sprinboardApi.createCustomerAddress = function (customerId, data, callback) {
    var data = data || {};
    request('customers/' + customerId + '/addresses', 'postJson', data, callback);
  };

  sprinboardApi.getCustomerAddresses = function (customerId, callback) {
    request('customers/' + customerId + '/addresses', 'get', {}, callback);
  };

  sprinboardApi.getCustomerAddress = function (customerId, addressId, callback) {
    request('customers/' + customerId + '/addresses/' + addressId, 'get', {}, callback);
  };

  sprinboardApi.updateCustomerAddress = function (customerId, addressId, data, callback) {
    var data = data || {};
    request('customers/' + customerId + '/addresses/' + addressId, 'putJson', data, callback);
  };


  // Inventory Values
  sprinboardApi.getInventoryValues = function (data, callback) {
    data = data || {};
    request('inventory/values', 'get', data, callback);
  };

  sprinboardApi.getInventoryValuesByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getInventoryValuesByFilterRaw, convertArgsToArray(arguments));
  };

  function getInventoryValuesByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('inventory/values?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber + '&group[]=location_id&group[]=item_id', 'get', {}, pager);
  };

  sprinboardApi.getInventoryValuesByLocation = function (itemId, data, callback) {
    data = data || {};
    data.item_id = itemId;
    data['group[]'] = 'location_id';
    data['_include[]'] = 'location';
    request('inventory/values', 'get', data, callback)
  };


  // Inventory Adjustments
  sprinboardApi.getInventoryAdjustmentsByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getInventoryAdjustmentsByFilterRaw, convertArgsToArray(arguments));
  };

  function getInventoryAdjustmentsByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('inventory/adjustments?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };

  sprinboardApi.createInventoryAdjustment = function (locationId, itemId, adjustmentReasonId, qtyAdjust, cost, data, callback) {
    data = data || {};
    data.location_id = locationId;
    data.item_id = itemId;
    data.adjustment_reason_id = adjustmentReasonId;
    data.qty = qtyAdjust;
    data.unit_cost = cost;
    request('inventory/adjustments', 'postJson', data, callback);
  }


  // Sales Ticket
  //   id: integer(required)
  //   type: text(required)
  //   station_id: integer
  //   metadata: text
  //   customer_id: integer
  //   source_location_id: integer
  //   sales_rep: text
  //   created_at: DateTime The date and time that this record was created
  //   updated_at: DateTime The date and time that this record was last updated
  //   public_id: text
  //   parent_transaction_id: integer
  //   recalculate: boolean
  //   status: text
  //   order_id: integer
  //   total: numeric(10, 2)
  //   created_by_user_id: integer
  //   completed_at: DateTime The date and time that this ticket was completed
  //   affect_inventory: boolean Whether or not the ticket should affect inventory on completion

  sprinboardApi.createTicket = function (data, callback) {  // TODO : verify required fields
    data = data || {};
    request('sales/tickets', 'postJson', data, callback);
  };

  sprinboardApi.createTicketLine = function (ticketId, data, callback) {
    data = data || {};
    request('sales/tickets/' + ticketId + '/lines', 'postJson', data, callback);
  };

  sprinboardApi.getTicket = function (id, callback) {
    request('sales/tickets/' + id + '?_include[]=customer', 'get', {}, callback);
  };

  sprinboardApi.getTicketLines = function (id, callback) {
    request('sales/tickets/' + id + '/lines?_include[]=item&per_page=1000', 'get', {}, callback);
  };

  sprinboardApi.getTicketsByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getTicketsByFilterRaw, convertArgsToArray(arguments));
  };

  function getTicketsByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('sales/tickets?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber + "&_include[]=customer", 'get', {}, pager);
  };

  sprinboardApi.updateTicket = function (id, data, callback) {
    request('sales/tickets/' + id, 'putJson', data, callback);
  };


  // Purchasing Vendor
  sprinboardApi.createVendor = function (name, data, callback) {
    data = data || {};
    data.name = name;
    request('purchasing/vendors', 'postJson', data, callback);
  };

  sprinboardApi.createVendorAddress = function (vendorId, city, state, country, postalCode, data, callback) {
    data = data || {};
    data.type = 'location'; // Vendor addresses are all of type location
    data.city = city;
    data.state = state;
    data.country = country;
    data.postal_code = postalCode;
    request('purchasing/vendors/' + vendorId + '/address', 'postJson', data, callback);
  };

  sprinboardApi.createVendorContact = function (vendorId, data, callback) {
    request('purchasing/vendors/' + vendorId + '/contact', 'postJson', data, callback);
  };

  sprinboardApi.getVendor = function (id, callback) {
    request('purchasing/vendors/' + id, 'get', {}, callback);
  };

  sprinboardApi.getVendorsByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getVendorsByFilterRaw, convertArgsToArray(arguments));
  };

  function getVendorsByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('purchasing/vendors?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };

  sprinboardApi.searchVendors = function (searchTerm, page, perPage, callback) {
    data = { 'query': searchTerm.replace(' ', '+'), 'page': page, 'per_page': perPage };
    request('vendors', 'get', data, callback);
  };

  sprinboardApi.updateVendor = function (id, data, callback) {
    request('purchasing/vendors/' + id, 'putJson', data, callback);
  };

  sprinboardApi.updateVendorAddress = function (vendorId, data, callback) {
    request('purchasing/vendors/' + vendorId + '/address', 'putJson', data, callback);
  };


  //Custom Fields
  sprinboardApi.getCustomField = function (id, callback) {
    request('custom_fields/' + id, 'get', {}, callback);
  };

  sprinboardApi.getCustomFieldsByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getCustomFieldsByFilterRaw, convertArgsToArray(arguments));
  };

  function getCustomFieldsByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('custom_fields?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };


  // Reporting
  sprinboardApi.getDailyItemMovement = function (jsonFilter, callback) {
    filterPager = new Pager(getDailyItemMovementRaw, convertArgsToArray(arguments));
  };

  function getDailyItemMovementRaw(pageNumber, perPage, jsonFilter, pager) {
    request('reporting/analyzer?metrics[]=source_sales.net_sales&metrics[]=purchasing_net.delta_qty&metrics[]=inventory_adjustment.delta_qty&metrics[]=inventory_transfer_received.delta_qty&metrics[]=inventory_transfer_shipped.delta_qty&metrics[]=source_sales.total_cost&metrics[]=source_sales.gross_margin_ratio&metrics[]=source_sales.net_qty_sold&metrics[]=source_sales.gross_margin&metrics[]=source_sales.gross_returns&groups[]=date.date&groups[]=location.id&groups[]=item.id&groups[]=item.custom@dynamic_gp&start_date=' + encodeURIComponent(jsonFilter.start_date) + '&end_date=' + encodeURIComponent(jsonFilter.end_date) + '&per_page=' + perPage + '&page=' + pageNumber, 'get', {}, pager);
  };

  sprinboardApi.getDailyTicketLines = function (jsonFilter, callback) {
    filterPager = new Pager(getDailyTicketLinesRaw, convertArgsToArray(arguments));
  };

  function getDailyTicketLinesRaw(pageNumber, perPage, jsonFilter, pager) {
    request('reporting/analyzer?metrics[]=source_sales.net_sales&metrics[]=source_sales.total_cost&metrics[]=source_sales.gross_margin_ratio&metrics[]=source_sales.net_qty_sold&metrics[]=source_sales.gross_margin&metrics[]=source_sales.gross_returns&groups[]=date.date&groups[]=location.id&groups[]=item.id&groups[]=sales_ticket.number&groups[]=sales_ticket.id&start_date=' + encodeURIComponent(jsonFilter.start_date) + '&end_date=' + encodeURIComponent(jsonFilter.end_date) + '&per_page=' + perPage + '&page=' + pageNumber, 'get', {}, pager);
  };


  // Gift Cards
  // TODO test this function
  //sprinboardApi.getGiftCard = function (giftCardNumber, callback) {
  //  request('gift_cards/' + giftCardNumber, 'get', {}, callback);
  //};


  // Purchasing Order
  //   id: integer(required)
  //   vendor_id: integer(required)
  //   status: text(required)
  //   receive_at_location_id: integer(required)
  //   public_id: text
  //   start_shipments_at: Date
  //   end_shipments_at: Date
  //   created_at: DateTime The date and time that this record was created
  //   updated_at: DateTime The date and time that this record was last updated
  //   created_by_user_id: integer
  //   description: text
  //   custom: object
  //   total_qty: numeric(10, 2)
  //   total_cost: numeric(10, 2)
  //   total_price: numeric(10, 2)
  //   total_received_qty: numeric(10, 2)
  //   total_received_cost: numeric(10, 2)
  //   total_received_price: numeric(10, 2)
  //   total_open_qty: numeric(10, 2)
  //   total_open_cost: numeric(10, 2)
  //   total_open_price: numeric(10, 2)
  //   initial_margin_ratio: numeric
  //   po_import_batch_id: integer

  sprinboardApi.getPurchaseOrder = function (id, callback) {
    request('purchasing/orders/' + id, 'get', {}, callback);
  };

  sprinboardApi.getPurchaseOrderLines = function (id, callback) {
    request('purchasing/orders/' + id + '/lines?per_page=1000', 'get', {}, callback);
  };

  sprinboardApi.getPurchaseOrdersByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getPurchaseOrdersByFilterRaw, convertArgsToArray(arguments));
  };

  function getPurchaseOrdersByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('purchasing/orders?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };


  // Templates
  sprinboardApi.getTemplate = function (id, callback) {
    request('templates/' + id + '.html.liquid', 'get', {}, callback);
  };

  sprinboardApi.updateTemplate = function (id, data, callback) {
    request('templates/' + id + '.html.liquid', 'putJson', data, callback);
  };


  // Users
  sprinboardApi.getUsersByFilter = function (jsonFilter, callback) {
    filterPager = new Pager(getUsersByFilterRaw, convertArgsToArray(arguments));
  };

  function getUsersByFilterRaw(pageNumber, perPage, jsonFilter, pager) {
    request('users?_filter=' + encodeURIComponent(JSON.stringify(jsonFilter)) + "&per_page=" + perPage + "&page=" + pageNumber, 'get', {}, pager);
  };


  // Custom Urls
  sprinboardApi.updateCustomUrls = function (urls, callback) {
    request('settings/custom_javascript_urls', 'putJson', { 'value': urls }, callback);
  };


  return sprinboardApi;
}

function convertArgsToArray(toConvert) {
  var arr = [];
  for (var i = 0; i < toConvert.length; i++) {
    arr.push(toConvert[i]);
  }

  return arr;
}